////////////////////////////////Lunar Death//////////////////////////////////
//////////////////////Coded and Designed by Mike Farrell/////////////////////

//System Includes////////////////////////////////////////////////////////////
#include <iostream.h>
#include <math.h>
#include <string.h>

//User Includes//////////////////////////////////////////////////////////////
#include "allegro.h"
#include "lander.h"

//Enums//////////////////////////////////////////////////////////////////////
enum flame_direction { DOWN, LEFT, RIGHT, NONE };

//Classes & Structs//////////////////////////////////////////////////////////
struct SHIP
{
  BITMAP *flame;
  float x, y, dx, dy;
  short fuel;
  flame_direction flame_dir;
  char flags[1], alive, down;

  SHIP(float x, float y, float dx = 0, float dy = 0, int fuel = 500);
  void set(float x, float y, float dx = 0, float dy = 0, int fuel = 500);
  void draw();
  void processing(int mode = 1);
  void kill();
};

struct TYPECHAR
{
  char c;
  unsigned short a;
  short x, y;
};

//Globals////////////////////////////////////////////////////////////////////
DATAFILE *data;
BITMAP *buffer;
SHIP ship(0, 0);
TYPECHAR t_string[4][40];
int typechar_pos = 0, typechar_done = TRUE, t_string_pos = 0;
volatile int fps;
int fpscounter, prologue_flag = FALSE;
char lives = 3;

//Prototypes/////////////////////////////////////////////////////////////////
void init();
int title_screen();
void prologue();
void play_game();
void shutdown();
void fatal_error(unsigned char err_code = 0);
void fps_proc();
void init_tchar_string(char str[40], int x, int y);
void typechar_processing();
void draw_typechar(int id);
void game_over();

//Callback Functions/////////////////////////////////////////////////////////
void fps_proc()
{
  fps++;
}
END_OF_FUNCTION(fps_proc);

//Member Functions///////////////////////////////////////////////////////////
SHIP::SHIP(float x, float y, float dx, float dy, int fuel)
{
  set(x, y, dx, dy, fuel);
}

void SHIP::set(float x, float y, float dx, float dy, int fuel)
{
  this->x = x;
  this->y = y;
  this->dx = dx;
  this->dy = dy;
  this->fuel = fuel;
  flame = NULL;
  flame_dir = DOWN;
  flags[0] = FALSE;
  alive = TRUE;
  down = FALSE;
}

void SHIP::draw()
{
  if(alive)
  {
    draw_sprite(buffer, (BITMAP *)data[lander].dat, (int)x, (int)y);
    if(flame && !down) switch(flame_dir)
    {
      case DOWN:
        draw_sprite(buffer, flame, (int)x+3, (int)y+14);
      break;
      case RIGHT:
        draw_sprite(buffer, (BITMAP *)data[fire_rt].dat, (int)x+11, (int)y+5);
      break;
      case LEFT:
        draw_sprite(buffer, (BITMAP *)data[fire_lt].dat, (int)x-4, (int)y+5);
      break;
      case NONE:
      break;
    }
  }
  else
  {
    draw_sprite(buffer, (BITMAP *)data[dead_lander].dat, (int)x, (int)y);    
  }
}

void SHIP::processing(int mode)
{
  static float t = 0.0;
  unsigned char i, j;
  char coll = FALSE;

  if(!alive || down) return;

  x += dx;
  y += dy;

  if(fpscounter % 5 == 0)
  {
    if(flame == (BITMAP *)data[fire_dn].dat)
      flame = (BITMAP *)data[fire_dnb].dat;
    else flame = (BITMAP *)data[fire_dn].dat;
  }

  switch(mode)
  {
    //Prologue
    case 0:
      flame_dir = DOWN;
      if(!flags[0])
      {
        if(y <= 120)
        {
          if(dy < 0) dy += 0.1;
          else { flags[0] = TRUE; dy = 1; }
        }
      }
      else
      {
        if(!prologue_flag) prologue_flag = TRUE;
        t += 0.1;
        y = 102+(sin(t)*5.0);
      }
    break;
    //Actual game
    case 1:
      if(x < -11) x = 320;
      if(x > 320) x = -11;
      if(y < 0) dy += 0.5;
      if(ship.fuel < 0) ship.fuel = 0;

      //really cheap and inefficient object collision
      for(j = 0; j < 14; j++) for(i = 0; i < 11; i++)
      {
        if(getpixel(buffer, (int)(x+i), (int)(y+j)) == makecol(128, 0, 0))
        {
          coll = TRUE;
          break;
        }
      }
      if(coll)
      {
        down = TRUE;
        if(dy > 1.25) kill();
        dx = dy = 0;
        clear_keybuf();
      }
    break;
  }
}

void SHIP::kill()
{
  alive = FALSE;
}

//Other Game Functions///////////////////////////////////////////////////////
void init()
{
  allegro_init();
  set_color_depth(24);
  install_keyboard();
  install_timer();
  if(set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0) != 0)
  {
    set_color_depth(32);
    if(set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0) != 0)
      fatal_error(2);
  }

  data = load_datafile("lander.dat");
  if(!data) fatal_error(1);
  buffer = create_bitmap(320, 240);
  if(!buffer) fatal_error();

  LOCK_FUNCTION(fps_proc);
  LOCK_VARIABLE(fps);
  if(install_int_ex(fps_proc, BPS_TO_TIMER(60)) < 0) fatal_error(3);
  text_mode(-1);
}

int title_screen()
{
  unsigned char screen_a = 0;
  short y = -87, curs_pos = 196, actual_pos = 196, go = TRUE;
  int ret = TRUE;

  fpscounter = fps = 0;

  while(go)
  {
    fpscounter++;
    if(key[KEY_ESC]) curs_pos = 212;
    if(key[KEY_ENTER] && screen_a == 255)
    {
      ret = (curs_pos == 196) ? TRUE : FALSE;
      go = FALSE;
    }
    if(key[KEY_DOWN] || key[KEY_UP])
    {
      if(curs_pos == 196) curs_pos = 212;
      else curs_pos = 196;
    }
    clear_keybuf();
    if(screen_a < 255) screen_a++;
    if(screen_a > 128 && y < 20) y++;
    if(actual_pos < curs_pos) actual_pos++;
    if(actual_pos > curs_pos) actual_pos--;

    if(fpscounter >= fps)
    {      
      blit((BITMAP *) data[back].dat, buffer, 0, 0, 0, 0, 320, 240);
      draw_sprite(buffer, (BITMAP *) data[title].dat, 87, y);
      if(screen_a == 255)
      {
        draw_sprite(buffer, (BITMAP *)data[lander].dat, 126, actual_pos);
        textout_centre(buffer, font, "Start", 161, 201, (curs_pos == 196) ? makecol(125, 0, 0) : 0);
        textout_centre(buffer, font, "Quit", 161, 217, (curs_pos == 212) ? makecol(125, 0, 0) : 0);
        textout_centre(buffer, font, "Start", 160, 200, (curs_pos == 196) ? makecol(255, 0, 0) : makecol(50, 50, 50));
        textout_centre(buffer, font, "Quit", 160, 216, (curs_pos == 212) ? makecol(255, 0, 0) : makecol(50, 50, 50));
      }
      if(screen_a < 255)
      {
        drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
        set_trans_blender(0, 0, 0, 255-screen_a);
        rectfill(buffer, 0, 0, 320, 240, 0);
        solid_mode();
      }
      blit(buffer, screen, 0, 0, 0, 0, 320, 240);
    }
    while(fpscounter > fps) {}
  }
  return ret;
}

void prologue()
{
  BITMAP *prologue_back = create_bitmap(320, 240);
  short x, y, ht = 0, go = TRUE, go_down = FALSE;
  float wt = 0;
  unsigned char screen_a = 255;

  if(!prologue_back) fatal_error();

  for(y = 0; y < 300; y+=100) for(x = 0; x < 400; x+=100)
    blit((BITMAP *) data[back].dat, prologue_back, 220, 0, x, y, 100, 100);
  x = 50;
  y = 75;

  ship.set(148, 240, 0, -2);

  fpscounter = fps = 0;
  clear_keybuf();
  init_tchar_string("This is Commodore, I am running low", 5, 150);
  while(go)
  {
    fpscounter++;

    if(keypressed()) go = FALSE;
    if(!go_down) ship.processing(0);
    else
    {
      ship.y += 2;
      screen_a -= 2;
      if(screen_a < 2) go = FALSE;
    }
    if(prologue_flag)
    {
      if(ht < 100)
      {
        ht++;
        wt += (73.0/100.0);
      }
      else if(!go_down)
      {
        if(typechar_done)
        {
          t_string_pos++;
          switch(t_string_pos)
          {
            case 1:
              init_tchar_string("on fuel.  I'm going to try to land", 5, 160);
            break;
            case 2:
              init_tchar_string("on a nearby planet...", 5, 170);
            break;
            case 3:
              init_tchar_string("..I'm going in.....", 5, 180);
            break;
            case 4:
              go_down = TRUE;               
            break;
          }
        }

        typechar_processing();
      }
    }    
    if(fpscounter >= fps)
    {
      int i;

      blit(prologue_back, buffer, 0, 0, 0, 0, 320, 240);
      if(prologue_flag) stretch_blit((BITMAP *)data[man].dat, buffer, 0, 0, 73, 100, (int)(x-(wt/2)), y-(ht/2), (int)wt, ht);
      ship.draw();
      if(ht >= 100) for(i = 0; i <= t_string_pos; i++)
        draw_typechar(i);
      if(screen_a < 255)
      {
        drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
        set_trans_blender(0, 0, 0, 255-screen_a);
        rectfill(buffer, 0, 0, 320, 240, 0);
        solid_mode();
      }
      blit(buffer, screen, 0, 0, 0, 0, 320, 240);
    }
    while(fpscounter > fps) {}
  }
  destroy_bitmap(prologue_back);
}

void game_over()
{
  BITMAP *prologue_back = create_bitmap(320, 240);
  short x, y, ht = 0, go = TRUE;
  float wt = 0;
  unsigned char screen_a = 255;

  if(!prologue_back) fatal_error();

  for(y = 0; y < 300; y+=100) for(x = 0; x < 400; x+=100)
    blit((BITMAP *) data[back].dat, prologue_back, 220, 0, x, y, 100, 100);
  x = 50;
  y = 75;

  fpscounter = fps = 0;
  clear_keybuf();
  typechar_pos = t_string_pos = 0;
  typechar_done = TRUE;
  init_tchar_string("NAHALHALHALAHALHHL!!!!!!!!!!!", 5, 150);
  while(go)
  {
    fpscounter++;

    if(keypressed()) go = FALSE;
    if(ht < 100)
    {
      ht++;
      wt += (73.0/100.0);
    }
    else
    {
      if(typechar_done)
      {
        t_string_pos++;
        switch(t_string_pos)
        {
          case 1:
            init_tchar_string("........Help me, man!", 5, 160);
          break;
          case 2:
            init_tchar_string("whoahaohoaa..halahlah!", 5, 170);
          break;
          case 3:
            init_tchar_string("........nahlahlahal!", 5, 180);
          break;
        }
      }
      if(t_string_pos < 4) typechar_processing();
    }
    if(fpscounter >= fps)
    {
      int i;

      blit(prologue_back, buffer, 0, 0, 0, 0, 320, 240);
      stretch_blit((BITMAP *)data[man_b].dat, buffer, 0, 0, 73, 100, (int)(x-(wt/2)), y-(ht/2), (int)wt, ht);
      if(t_string_pos < 4) if(ht >= 100) for(i = 0; i <= t_string_pos; i++)
        draw_typechar(i);
      if(screen_a < 255)
      {
        drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
        set_trans_blender(0, 0, 0, 255-screen_a);
        rectfill(buffer, 0, 0, 320, 240, 0);
        solid_mode();
      }
      blit(buffer, screen, 0, 0, 0, 0, 320, 240);
    }
    while(fpscounter > fps) {}
  }
  destroy_bitmap(prologue_back);
}


void play_game()
{
  short go = TRUE;

  fps = fpscounter = 0;
  ship.set(160, -16, 0, 1.5);
  clear_keybuf();
  while(go)
  {
    fpscounter++;

    //Input
    if(!ship.alive)
    {
      if(keypressed()) go = FALSE;      
    }
    else
    {
      if(key[KEY_ESC]) go = FALSE;
      if(key[KEY_UP] && ship.fuel > 0)
      {
        ship.fuel--;
        ship.flame_dir = DOWN;
        if(ship.dy > -2.0) ship.dy -= 0.2;
      }
      if(key[KEY_RIGHT] && ship.fuel > 0)
      {
        ship.fuel--;
        ship.flame_dir = LEFT;
        if(ship.dx < 2.0) ship.dx += 0.2;
      }
      if(key[KEY_LEFT] && ship.fuel > 0)
      {
        ship.fuel--;
        ship.flame_dir = RIGHT;
        if(ship.dx > -2.0) ship.dx -= 0.2;
      }
      if((!key[KEY_LEFT] && !key[KEY_RIGHT] && !key[KEY_UP]) || (ship.fuel == 0.0))
      {
        ship.flame_dir = NONE;
        if(ship.dy < 3.0) ship.dy += 0.2;
  //      ship.dx += (ship.dx < 0) ? 0.1 : -0.1;
      }
    }
    
    //Process
    ship.processing();

    //Output
    if(fpscounter >= fps)
    {      
      blit((BITMAP *)data[back].dat, buffer, 0, 0, 0, 0, 320, 240);
      ship.draw();
      textprintf(buffer, font, 0, 12, makecol(255, 0, 0), "Lives: %d", (int)lives);
      textprintf(buffer, font, 0, 0, makecol(255-(ship.fuel/2), ship.fuel/2, 0), "Fuel:  %d", ship.fuel);
      blit(buffer, screen, 0, 0, 0, 0, 320, 240);
    }
    while(fpscounter > fps) {}
  }
}

void shutdown()
{
  remove_int(fps_proc);
  unload_datafile(data);
  destroy_bitmap(buffer);
  allegro_exit();
}

void fatal_error(unsigned char err_code)
{
  set_gfx_mode(GFX_TEXT, 80, 25, 0, 0);
  switch(err_code)
  {
    case 0:
      cout << "Out of memory!!\n\n";
    break;
    case 1:
      cout << "Error loading lander.dat!!!\n\n";
    break;
    case 2:
      cout << "Unable to find a suitable graphics mode!!\n";
    break;
    case 3:
      cout << "Error intializing timer!!!\n\n";
    break;
  }
  allegro_exit();
  exit(1);
}

void init_tchar_string(char str[40], int x, int y)
{
  unsigned int i;

  typechar_pos = typechar_done = 0;
  for(i = 0; i < strlen(str); i++)
  {
    t_string[t_string_pos][i].c = str[i];
    t_string[t_string_pos][i].a = 0;
    t_string[t_string_pos][i].x = x+(i*8);
    t_string[t_string_pos][i].y = y;
  }
  for(i = strlen(str); i < 40; i++)
  {         
    t_string[t_string_pos][i].c = NULL;
  } 
}

void typechar_processing()
{
  t_string[t_string_pos][typechar_pos].a+=50;
  if(t_string[t_string_pos][typechar_pos].a > 255)
  {
    t_string[t_string_pos][typechar_pos].a = 255;
    typechar_pos++;
  }
  if(!t_string[t_string_pos][typechar_pos].c) typechar_done = TRUE;
}

void draw_typechar(int id)
{
  int i;

  for(i = 0; i < 40; i++) if(t_string[id][i].c)
    textprintf(buffer, font, t_string[id][i].x, t_string[id][i].y, makecol(t_string[id][i].a, t_string[id][i].a, t_string[id][i].a), "%c", t_string[id][i].c);
}

//Main///////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[]) {

#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif


  init();
  if(title_screen())
  {
    prologue();
    do
    {     
      play_game();
      lives--;
      if(lives <= 0 && !ship.alive) { game_over(); break; }
    } while(!ship.alive);
  }
  shutdown();
  return 0;
}

//Allegro 3.12 users COMMENT THIS NEXT LINE!
END_OF_MAIN();
