
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "allegro.h"
#include "input.h"
#include "game.h"
#include "log.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif



#define NUM_KEYS 5

char *control_name[NUM_KEYS] = {"Up", "Down", "Right", "Left", "Fire"};

int input_names[NUM_KEYS] = {INPUT_KEY_UP, INPUT_KEY_DOWN, INPUT_KEY_RIGHT, INPUT_KEY_LEFT, INPUT_KEY_FIRE};
const char *default_key[NUM_PLAYERS][NUM_KEYS] = {
	{"Up", "Down", "Right", "Left", "RControl"},
	{"W", "S", "D", "A", "LControl"}
};


INPUT *load_player_input_config(int n) {

	char keyname[40];
	const char *type;
	int itype = 0;
	int i;

	INPUT *ret;

	if (n < 1 || n > NUM_PLAYERS) {
		LOG("--== Error ==-- Config::load_player_input_config: Invalid player number (%i)\n", n);
		return NULL;
	}

	set_config_file("zasx.cfg");

	sprintf(keyname, "Player%i_Type", n);
	type = get_config_string("controls", keyname, "keyboard");

	if (stricmp(type, "keyboard") == 0)
		itype = INPUT_TYPE_KEYBOARD;
	else if (stricmp(type, "joystick") == 0)
		itype = INPUT_TYPE_JOYSTICK;

	if (itype) {
		const char *new_key;
		ret = create_input(itype);

		for (i = 0; i < NUM_KEYS; i++) {
			sprintf(keyname, "Player%i_%s", n, control_name[i]);
			new_key = get_config_string("controls", keyname, default_key[n - 1][i]);
			set_input(ret, input_names[i], new_key);
		}
	}
	else {
		LOG("--== Warning ==-- Config::load_player_input_config: Unknown input device (%s). Assuming default key bindings.\n", type);
		ret = create_input(INPUT_TYPE_KEYBOARD);
		for (i = 0; i < NUM_KEYS; i++) {
			set_input(ret, input_names[i], default_key[n - 1][i]);
		}
	}

	return ret;
}


void save_player_config(int n, INPUT *in) {

	int i;
	char keyname[40];	
	char buf[15];
	const char *type;

	if (n < 1 || n > NUM_PLAYERS) {
		LOG("--== Error ==-- Config::save_player_input_config: Invalid player number (%i)\n", n);
		return;
	}

	if (!in) {
		LOG("--== Error ==-- Config::save_player_input_config: Null argument 'in'\n");
		return;
	}

	set_config_file("zasx.cfg");

	sprintf(keyname, "Player%i_Type", n);
	type = get_input_type_name(in);

	set_config_string("controls", keyname, type);

	for (i = 0; i < NUM_KEYS; i++) {
		sprintf(keyname, "Player%i_%s", n, control_name[i]);
		set_config_string("controls", keyname, get_input_name(in, input_names[i], buf, 11));
	}		

	return;
}

void get_default_player_config(int n, INPUT *dev) {

	int i;

	if (!dev) {
		LOG("--== Error ==-- Config::get_default_player_config: 'dev' is NULL.\n");
		return;
	}

	if (n < 1 || n > NUM_PLAYERS) {
		LOG("--== Error ==-- Config::get_default_player_config: Invalid player number (%i)\n", n);
		return;
	}

	for (i = 0; i < NUM_KEYS; i++)
		set_input(dev, input_names[i], default_key[n - 1][i]);

	return;
}