
#include <string.h>

#include "allegro.h"
#include "gfx.h"
#include "gametimer.h"
#include "particle.h"
#include "effect.h"
#include "game.h"
#include "drs.h"
#include "log.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif


EFFECT **Effect = NULL;
int NumEffects = 0;
static int ArraySize = 0;
static int EffectInstalled = 0;

void add_effect(PARTICLE *p, float x, float y, int max_rad, int max_col, int min_col, int way, int speed) {

	EFFECT *e = malloc(sizeof(EFFECT));

	if (!e) {
		LOG("--== Error ==-- Effect::add_effect: Ran out of memory while trying to allocate %i bytes.\n", sizeof(EFFECT));
		return;
	}

	memset(e, 0, sizeof(EFFECT));

	e->particle_follow = p;
	e->x = x; e->y = y;
	e->max_rad = max_rad;
	e->max_col = max_col;
	e->min_col = min_col;
	e->way = way;
	e->speed = speed;

	if (ArraySize - NumEffects <= 1) {
		EFFECT **ef = realloc(Effect, sizeof(EFFECT*) * (ArraySize + 16));

		if (!ef) {
			LOG("--== Error ==-- Effect::add_effect: Ran out of memory while trying to allocate %i bytes.\n", sizeof(EFFECT*) * (ArraySize + 16));
			free(e);
			return;
		}
		Effect = ef;
		ArraySize += 16;
	}

	Effect[NumEffects] = e;

	NumEffects++;

	return;
}

static void remove_effect(EFFECT *e) {

	int i;

	for (i = 0; i < NumEffects; i++) {
		if (Effect[i] == e) {		
			Effect[i] = Effect[NumEffects - 1];
			NumEffects--;
			break;
		}
	}

	if (NumEffects == 0) {
		if (Effect)
			free(Effect);
		Effect = NULL;
		ArraySize = 0;
	}
	else if (ArraySize - NumEffects > 32) {
		Effect = realloc(Effect, sizeof(EFFECT*) * NumEffects);
		ArraySize = NumEffects;
	}
	return;
}


void destroy_effect(EFFECT *e) {

	if (!e)
		return;

	remove_effect(e);

	free(e);
}

void draw_effects(BITMAP *dest) {

	int i;

	if (!dest) {
		LOG("--== Error ==-- Effect::draw_effects: Parameter 'dest' is NULL.\n");
		return;
	}

	if (_color_depth == 8)
		color_map = &transmaparray[3];
	else
		set_add_blender(128, 128, 128, 255);

	drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);

	for (i = 0; i < NumEffects; i++) {
		EFFECT *e = Effect[i];

		int x, y, r, c;

		if (!e)
			continue;

		if (e->particle_follow) {
			x = (int)e->particle_follow->x + e->x;
			y = (int)e->particle_follow->y + e->y;
		}
		else {
			x = e->x;
			y = e->y;
		}
		if (e->way)
			r = (1000 - MID(0, e->step, 1000)) * e->max_rad / 1000.0;
		else
			r = MID(0, e->step, 1000) * e->max_rad / 1000.0;

		c = makecol((int)((getr(e->max_col) * e->step + getr(e->min_col) * (1000 - e->step)) / 1000.0),
			(int)((getg(e->max_col) * e->step + getg(e->min_col) * (1000 - e->step)) / 1000.0),
			(int)((getb(e->max_col) * e->step + getb(e->min_col) * (1000 - e->step)) / 1000.0));

		circlefill(dest, x, y, r, c);
		circlefill(dest, x - DISPLAY_W, y, r, c);
		circlefill(dest, x, y - DISPLAY_H, r, c);
		circlefill(dest, x + DISPLAY_W, y, r, c);
		circlefill(dest, x, y + DISPLAY_H, r, c);

		{
			int w, h;
			x = x - r - 3;
			y = y - r - 3;
			w = 2 * r + 6;
			h = 2 * r + 6;

			DRS_add_rectangle(x, y, w, h);
			DRS_add_rectangle(x + DISPLAY_W, y, w, h);
			DRS_add_rectangle(x - DISPLAY_W, y, w, h);
			DRS_add_rectangle(x, y + DISPLAY_H, w, h);
			DRS_add_rectangle(x, y - DISPLAY_H, w, h);
			DRS_add_rectangle(x, y, w, h);

			DRS_add_rectangle(e->old_x, e->old_y, e->old_w, e->old_h);
			DRS_add_rectangle(e->old_x + DISPLAY_W, e->old_y, e->old_w, e->old_h);
			DRS_add_rectangle(e->old_x - DISPLAY_W, e->old_y, e->old_w, e->old_h);
			DRS_add_rectangle(e->old_x, e->old_y + DISPLAY_H, e->old_w, e->old_h);
			DRS_add_rectangle(e->old_x, e->old_y - DISPLAY_H, e->old_w, e->old_h);
			
			e->old_x = x;
			e->old_y = y;
			e->old_w = w;
			e->old_h = h;

		}
	}
	drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);

	return;
}

int process_effects(void *dummy) {

	int i;

	if (GamePaused)
		return 0;

	for (i = 0; i < NumEffects; i++) {
		EFFECT *e = Effect[i];

		if (!e)
			continue;

		e->step += e->speed;

		if (e->step >= 1000) {
			destroy_effect(e);
			i--;
			continue;
		}
	}
	return 0;
}

int install_effects() {

	if (EffectInstalled)
		return 0;

	GTIME_AddFunction(process_effects, 10, NULL, GTIME_NORMAL);
	EffectInstalled = 1;

	return 0;
}

void clear_effects() {

	while (NumEffects)
		remove_effect(Effect[0]);

	NumEffects = 0;

	return;
}
