#include <math.h>
#include <time.h>
#include <string.h>
#include <stdio.h>

#include "allegro.h"
//#include "jgmod.h"
#include "particle.h"
#include "asteroid.h"
#include "star.h"
#include "ppcol.h"
#include "game.h"
#include "scorer.h"
#include "log.h"
#include "effect.h"
#include "weapon.h"
#include "player.h"
#include "input.h"
#include "gfx.h"
#include "dataload.h"
#include "anim.h"
#include "gamemenu.h"
#include "gametimer.h"
#include "drs.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif


#define DIST(x, y) sqrt((x)*(x) + (y)*(y))

int ShowFPS = 0;

BITMAP *Ast[9];
BITMAP *BackBuffer;
DATAFILE *demodata;
BITMAP *Smoke, *Bullet, *Dust;
BITMAP *PlayerBitmap[NUM_PLAYERS];
WEAPON *NormalGun, *LaserGun, *RapidFireGun, *Laser2Gun, *Laser3Gun;

FONT *DigitFont = NULL;

SAMPLE *Boom, *Tick, *Engine, *PowerUp;
// JGMOD *Music = NULL;

GAME Game;

COLOR_MAP transmap;
COLOR_MAP transmaparray[4];

RGB_MAP palette_map;

RGB* GamePal;

int GamePaused = 0;

#ifndef M_PI
#define M_PI 3.14159
#endif


void InitLevel(int lev) {

	int i, j;

	/* Build up new data */
	Game.Asteroid = malloc(sizeof(ASTEROID**) * lev);

	if (!Game.Asteroid) {
		LOG("--== Error ==-- Main::InitLevel: Ran out of memory while allocating %i bytes.\n", sizeof(ASTEROID**) * lev);
		return;
	}

	Game.NumAsteroids = lev;
	for (i = 0; i < lev; i++) {
		ASTEROID *a;
		Game.Asteroid[i] = create_asteroid(rand() % 3, 0);
		a = Game.Asteroid[i];

		if (!a)
			continue;

		add_particle_bitmap(a->Particle, Ast[a->family * 3 + a->size]);
		for (j = 0; j < NUM_PLAYERS; j++) {
			if (!Game.Player[j])
				continue;
			while (DIST(a->Particle->x - Game.Player[j]->Particle->x, a->Particle->y - Game.Player[j]->Particle->y) < 80)
				set_particle_coord(a->Particle, rand() % DISPLAY_W, rand() % DISPLAY_H);		
		}
	}

	/* Show that we changed levels */
	Game.LevelSwitchDelay = 1500;
	Game.LevelSwitchDelta = 0;
	Game.Level = lev;
	for (j = 0; j < NUM_PLAYERS; j++) {
		if (Game.Player[j] && get_player_property(Game.Player[j], PLAYER_ALIVE)) {
			set_player_property(Game.Player[j], PLAYER_INVINCIBILITY);
			set_player_invincibility(Game.Player[j], 200);
		}
	}

	Game.TimeToLive = 0x7FFFFFFF;

	play_sample(Boom, 255, 128, 2000, FALSE);

	return;
}



void DustBlast(PARTICLE *a, int num) {

	int j;

	for (j = 0; j < num; j++) {
			
		PARTICLE *n = create_particle();
		float angle;						

		if (!n)
			continue;

		angle = (rand() % 360) * 3.1415 / 180;
		set_particle_coord(n, a->x + (rand() % 200 - 100) / 10.0, a->y + (rand() % 200 - 100) / 10.0);
		set_particle_velocity(n, cos(angle) * (rand() % 80), sin(angle) * (rand() % 80));
		add_particle_bitmap(n, Dust);
		set_particle_property(n, PARTICLE_ALPHA | PARTICLE_ALIVE | PARTICLE_ALPHA_DIMINISH);
		n->life = rand() % 120;
		n->alpha_speed = -2;
		n->type = 'D';
	}
	return;
}


int Collide(PARTICLE *p1, BITMAP *s1, PARTICLE *p2, BITMAP *s2) {

	int temp_w, temp_h;
	int x1, y1, x2, y2;
	BITMAP *ps1, *ps2;

	if (!p1 || !p2)
		return 0;

	if (!p1->aux_bmp) {
		temp_w = get_particle_bitmap_w(p1);
		temp_h = get_particle_bitmap_h(p1);
	}
	else {
		temp_w = p1->aux_bmp->w;
		temp_h = p1->aux_bmp->h;
	}
	x1 = (int)p1->x - temp_w / 2;
	y1 = (int)p1->y - temp_h / 2;

	if (!p2->aux_bmp) {
		temp_w = get_particle_bitmap_w(p2);
		temp_h = get_particle_bitmap_h(p2);
	}
	else {
		temp_w = p2->aux_bmp->w;
		temp_h = p2->aux_bmp->h;
	}
	x2 = (int)p2->x - temp_w / 2;
	y2 = (int)p2->y - temp_h / 2;

	if (!s1)
		ps1 = get_particle_current_frame(p1);
	else
		ps1 = s1;

	if (!s2)
		ps2 = get_particle_current_frame(p2);
	else
		ps2 = s2;

	if (!ps1 || !ps2)
		return 0;

	return check_pp_collision_normal(ps1, ps2, x1, y1, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 - DISPLAY_W, y1, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 + DISPLAY_W, y1, x2, y2)
	    || check_pp_collision_normal(ps1, ps2, x1, y1 - DISPLAY_H, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 - DISPLAY_W, y1 - DISPLAY_H, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 + DISPLAY_W, y1 - DISPLAY_H, x2, y2)
	    || check_pp_collision_normal(ps1, ps2, x1, y1 + DISPLAY_H, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 - DISPLAY_W, y1 + DISPLAY_H, x2, y2)
		|| check_pp_collision_normal(ps1, ps2, x1 + DISPLAY_W, y1 + DISPLAY_H, x2, y2);
}

static ASTEROID* find_asteroid(PARTICLE *p) {
	int i;
	for (i = 0; i < Game.NumAsteroids; i++) {
		if (Game.Asteroid[i]->Particle == p)
			return Game.Asteroid[i];
	}
	return NULL;
}

static PLAYER* find_player(PARTICLE *p) {
	int i;
	for (i = 0; i < NUM_PLAYERS; i++) {
		if (Game.Player[i] && Game.Player[i]->Particle == p)
			return Game.Player[i];
	}
	return NULL;
}



int play_game(void* dummy) {

	static int smokecount = 0;

	int d, i, j, k;
	float vx, vy;

	if (GamePaused)
		return 0;
	
	if (Game.LevelSwitchDelay > 0 && Game.LevelSwitchDelay != 0x7FFFFFFF)
		Game.LevelSwitchDelay -= 20;

	if (Game.LevelSwitchDelay <= 0) {
		Game.LevelSwitchDelay = 0x7FFFFFFF;
		Game.LevelSwitchDelta = 1;
	}

	if (Game.TimeToLive > 0 && Game.TimeToLive != 0x7FFFFFFF)
		Game.TimeToLive -= 20;

	for (j = 0; j < NUM_PLAYERS; j++) {

		PLAYER *pl = Game.Player[j];

		if (!pl)
			continue;

		get_player_velocity(Game.Player[j], &vx, &vy);
		d = DIST(vx, vy);

		if (d < 10 && rand() % 5 < 2)
			continue;
	
		adjust_sample(Engine, (int)(d * 255 / 300), 128, 1000, FALSE);
	
		if (get_player_property(pl, PLAYER_ALIVE)) {
			float angle, x, y;
			int ph;

			PARTICLE *p = create_particle();
			set_particle_property(p, PARTICLE_ALIVE | PARTICLE_WRAP | PARTICLE_ALPHA_DIMINISH | PARTICLE_ALPHA);
	
			angle = get_player_angle(pl);
			get_player_coord(pl, &x, &y);

			ph = get_particle_bitmap_h(pl->Particle);
			set_particle_coord(p, x - sin(angle) * ph / 2, y + cos(angle) * ph / 2);
			set_particle_velocity(p, -sin(angle) * (150 + rand() % 100) + vx, cos(angle) * (150 + rand() % 100) + vy);
			set_particle_life(p, 5 + d / 10);
			add_particle_bitmap(p, Smoke);
			set_particle_alpha(p, 255);
			set_particle_type(p, 'F');
		}
	}

	if (rand() % 200 < 5)
		add_ray();

	/* Add a health power up */
	if ((rand() % 2000) < 3) {
		PARTICLE *p = make_anim_particle("Health");
		set_particle_property(p, PARTICLE_ALIVE | PARTICLE_WRAP | PARTICLE_COLLIDE | p->prop);
		set_particle_coord(p, rand() % DISPLAY_W, rand() % DISPLAY_H);
		set_particle_velocity(p, rand() % 20, rand() % 20);
		set_particle_rotation_speed(p, rand() % ftofix(0.05) - ftofix(0.025));
		set_particle_life(p, 1000);
		set_particle_alpha(p, 255);
		set_particle_type(p, 'h');
	}


	/* Check for collisions */
	for (i = 0; i < NumParticles; i++) {

		PARTICLE *p1 = Particle[i];
		BITMAP *p1frame;
		
		if (!p1)
			continue;

		if (!(p1->prop & PARTICLE_COLLIDE))
			continue;

		p1frame = get_particle_current_frame(p1);
		if (!p1frame)
			continue;
	

		for (j = i + 1; j < NumParticles; j++) {

			PARTICLE *p2 = Particle[j];			

			PLAYER *player = NULL;
			ASTEROID *asteroid = NULL;
			PARTICLE *powerup = NULL;
			PARTICLE *bullet = NULL;
			BITMAP *p2frame = NULL, *asteroid_frame = NULL, *player_frame = NULL, *bullet_frame = NULL, *powerup_frame = NULL;

			if (!p2)
				continue;

			if (!(p2->prop & PARTICLE_COLLIDE))
				continue;

			p2frame = get_particle_current_frame(p2);
			if (!p2frame)
				continue;

			/* No collision if of the same type */
			if (p1->type == p2->type)
				continue;

			if (p1->type == 'A') {
				asteroid = find_asteroid(p1);
				asteroid_frame = p1frame;
			}
			else if (p1->type == 'P') {
				player = find_player(p1);
				player_frame = p1frame;
			}
			else if (p1->type == 'r' || p1->type == 'h' || p1->type == 'l') {
				powerup = p1;
				powerup_frame = p1frame;
			}
			else if (p1->type == 'B') {
				bullet = p1;
				bullet_frame = p1frame;
			}

			if (p2->type == 'A') {
				asteroid = find_asteroid(p2);
				asteroid_frame = p2frame;
			}
			else if (p2->type == 'P') {
				player = find_player(p2);
				player_frame = p2frame;
			}
			else if (p2->type == 'r' || p2->type == 'h' || p2->type == 'l') {
				powerup = p2;
				powerup_frame = p2frame;
			}
			else if (p2->type == 'B') {
				bullet = p2;
				bullet_frame = p2frame;
			}

			/* Player-Asteroid collision */
			if (player && asteroid) {
				PARTICLE *ppl = player->Particle;

				if ((!get_player_property(player, PLAYER_ALIVE))
					|| get_player_property(player, PLAYER_INVINCIBILITY)
					|| (!Collide(ppl, player_frame, asteroid->Particle, asteroid_frame)))
					continue;

				set_player_hp(player, get_player_hp(player) - 1, -1);
				add_player_score(player, 5);
				play_sample(Boom, 255, 128, 1000, FALSE);

				/* Lose weapons */
				if (player_has_weapon(player, LaserGun) && get_player_weapon(player) == LaserGun) {
					remove_player_weapon(player, LaserGun);

					if (player_has_weapon(player, RapidFireGun))
						select_player_weapon(player, RapidFireGun);
					else
						select_player_weapon(player, NormalGun);
				}
				else if (player_has_weapon(player, Laser2Gun) && get_player_weapon(player) == Laser2Gun) {
					remove_player_weapon(player, Laser2Gun);
					add_player_weapon(player, LaserGun);
					select_player_weapon(player, LaserGun);
				}
				else if (player_has_weapon(player, Laser3Gun) && get_player_weapon(player) == Laser3Gun) {
					remove_player_weapon(player, Laser3Gun);
					add_player_weapon(player, Laser2Gun);
					select_player_weapon(player, Laser2Gun);
				}
	
				/* Player is dead? */
				if (get_player_hp(player) <= 0) {					

					/* blow up */
					ppl->prop |= PARTICLE_INVISIBLE;
					unset_player_property(player, PLAYER_ALIVE);
		
					play_sample(Boom, 255, 128, 1200, FALSE);
					play_sample(Boom, 255, 128, 900, FALSE);
					play_sample(Boom, 255, 128, 500, FALSE);

					add_effect(NULL, ppl->x + rand() % 40 - 20, ppl->y + rand() % 40 - 20, 200, makecol(0, 0, 0), makecol(255, 255, 32), 0, rand() % 6 + 2);
					add_effect(NULL, ppl->x + rand() % 40 - 20, ppl->y + rand() % 40 - 20, 200, makecol(0, 0, 0), makecol(255, 255, 32), 0, rand() % 6 + 2);
					add_effect(NULL, ppl->x, ppl->y, 200, makecol(0, 0, 0), makecol(255, 32, 32), 0, 8);
	
					for (k = 0; k < 100; k++) {
				
						PARTICLE *p = create_particle();
						float angle;
				
						if (!p)
							break;
									
						angle = (rand() % 360) * 3.1415 / 180;
						set_particle_coord(p, ppl->x + (rand() % 200 - 100) / 10.0, ppl->y + (rand() % 200 - 100) / 10.0);
						set_particle_velocity(p, cos(angle) * (rand() % 60), sin(angle) * (rand() % 60));
						add_particle_bitmap(p, Dust);
						set_particle_property(p, PARTICLE_ALPHA | PARTICLE_ALIVE);
						p->life = 150 + rand() % 150;
						p->type = 'E';						
					}
				}
				else {
					set_player_property(player, PLAYER_INVINCIBILITY);
					DustBlast(get_player_particle(player), 20);				
				}
			}
			else if (player && powerup) {
				PARTICLE *ppl = player->Particle;

				if ((!get_player_property(player, PLAYER_ALIVE))
					|| !Collide(ppl, player_frame, powerup, powerup_frame))
					continue;

				/* If it's a health, add 1 hp */
				if (powerup->type == 'h') {					
					play_sample(PowerUp, 255, 128, 1000, FALSE);
					set_player_hp(player, get_player_hp(player) + 1, -1);
					add_player_score(player, 25);
					add_effect(ppl, 0, 0, 150, makecol(0, 0, 0), makecol(255, 255, 32), 1, 10);

					if (get_player_hp(player) > get_player_maxhp(player) / 2) {
						play_sample(PowerUp, 255, 128, 900, FALSE);
						play_sample(PowerUp, 255, 128, 1100, FALSE);
						add_player_score(player, 200 * get_player_hp(player) / get_player_maxhp(player));
					}			
				}

				/* If it's a rapidfire, upgrade weapons */
				if (powerup->type == 'r') {
					play_sample(PowerUp, 255, 128, 1500, FALSE);
					/* Swicth weapons */
					remove_player_weapon(player, RapidFireGun);
					add_player_weapon(player, RapidFireGun);
					select_player_weapon(player, RapidFireGun);

					add_player_score(player, 125);
					add_effect(ppl, 0, 0, 150, makecol(0, 0, 0), makecol(255, 32, 32), 0, 8);
				}
				/* If it's a laser, upgrade weapons */
				else if (powerup->type == 'l') {
					play_sample(PowerUp, 255, 128, 1500, FALSE);

					/* Swicth weapons */
					if (player_has_weapon(player, Laser2Gun) || player_has_weapon(player, Laser3Gun)) {
						remove_player_weapon(player, Laser2Gun);
						remove_player_weapon(player, Laser3Gun);
						add_player_weapon(player, Laser3Gun);
						select_player_weapon(player, Laser3Gun);
					}
					else if (player_has_weapon(player, LaserGun)) {
						remove_player_weapon(player, LaserGun);
						remove_player_weapon(player, Laser2Gun);
						add_player_weapon(player, Laser2Gun);
						select_player_weapon(player, Laser2Gun);
					}
					else {
						add_player_weapon(player, LaserGun);
						select_player_weapon(player, LaserGun);
					}
					add_player_score(player, 125);
					add_effect(ppl, 0, 0, 150, makecol(0, 0, 0), makecol(255, 32, 32), 0, 8);
				}

				destroy_particle(powerup);

				if (powerup == p1) {
					i--;
					break;
				}
				else
					j--;

				continue;
			}
			if (asteroid && bullet) {

				ASTEROID *asteroid2;
				float angle;
				int reset = 0;

				if (!Collide(asteroid->Particle, asteroid_frame, bullet, bullet_frame))
					continue;

				asteroid->hp -= bullet->power;

				/* Collision response - asteroids changes rotations + velocity */
				{
					float px = asteroid->Particle->x - bullet->x;
					float py = asteroid->Particle->y - bullet->y;
					float bx = bullet->vx, by = bullet->vy;
					float d, rad, sign;

					rad = sqrt(px * px + py * py);
					if (!rad)
						rad = 1;
					px /= rad;  py /= rad;

					rad = sqrt(bx * bx + by * by);
					if (!rad)
						rad = 1;
					bx /= rad;  by /= rad;

					d = dot_product_f(px, py, 0, bx, by, 0);

					sign = px * by - py * bx;
					sign = ((sign < 0) ? -1 : 1);

					asteroid->Particle->vx += px * d * 4;
					asteroid->Particle->vy += py * d * 4;

					d = acos(d);

					asteroid->Particle->rot_speed += ftofix(d * -10 * sign);
				}


				play_sample(Boom, 255, 128, 1000, FALSE);

				DustBlast(bullet, fixtoi(bullet->power * 4 * (asteroid->size + 1)) + 5);

				do {

				if (asteroid->hp <= 0) {

					/* Asteroid too small? */					
					if (asteroid->size >= 2) {
						for (k = 0; k < Game.NumAsteroids; k++)
							if (asteroid == Game.Asteroid[k])
								break;
							
						destroy_asteroid(asteroid);

						if (k < Game.NumAsteroids) {
							Game.Asteroid[k] = Game.Asteroid[Game.NumAsteroids - 1];
							Game.NumAsteroids--;
						}

						reset = 1;

						if (asteroid_frame == p1frame)
							i--;

						break;
					}
						
	
					/* Split asteroid */				
					asteroid2 = create_asteroid(asteroid->family, asteroid->size + 1);

					asteroid->hp = itofix(1);
					asteroid->size++;
					remove_particle_bitmaps(asteroid->Particle);
					add_particle_bitmap(asteroid->Particle, Ast[asteroid2->family * 3 + asteroid2->size]);
					add_particle_bitmap(asteroid2->Particle, Ast[asteroid2->family * 3 + asteroid2->size]);
					//LOG("Asteroid split: original frame: 0x%X, family: %i, size: %i bitmap 0x%X, new_frame: 0x%X, 0x%X\n", asteroid_frame, asteroid2->family, asteroid2->size, Ast[asteroid2->family * 3 + asteroid2->size], get_particle_current_frame(asteroid->Particle), get_particle_current_frame(asteroid2->Particle));

					/* Adjust frames */
					if (p1frame == asteroid_frame)
						p1frame = get_particle_current_frame(asteroid->Particle);
					else
						p2frame = get_particle_current_frame(asteroid->Particle);
	
					angle = atan2(asteroid->Particle->vy, asteroid->Particle->vx);
					asteroid->Particle->vx += cos(angle + 3.1415/2) * 100;
					asteroid->Particle->vy += sin(angle + 3.1415/2) * 100;
						
					asteroid2->Particle->x = asteroid->Particle->x;
					asteroid2->Particle->y = asteroid->Particle->y;
					asteroid2->Particle->vx = asteroid->Particle->vx - 2 * cos(angle + 3.1415/2) * 100;
					asteroid2->Particle->vy = asteroid->Particle->vy - 2 * sin(angle + 3.1415/2) * 100;
					asteroid2->Particle->rot_speed = -asteroid->Particle->rot_speed;

					
					Game.Asteroid = realloc(Game.Asteroid, sizeof(ASTEROID*) * (Game.NumAsteroids + 1));
					Game.Asteroid[Game.NumAsteroids] = asteroid2;
					Game.NumAsteroids++;

					/* Add power ups */
					/* Add a rapid fire power up */
					if ((rand() % 4) < 1) {
						PARTICLE *p = make_anim_particle("RapidFire");
						set_particle_property(p, PARTICLE_ALIVE | PARTICLE_WRAP | PARTICLE_COLLIDE | p->prop);
						set_particle_coord(p, asteroid->Particle->x, asteroid->Particle->y);
						set_particle_velocity(p, rand() % 40, rand() % 40);
						set_particle_rotation_speed(p, rand() % ftofix(0.05) - ftofix(0.025));
						set_particle_life(p, 1000);
						set_particle_alpha(p, 255);
						set_particle_type(p, 'r');
					}
					/* Add a laser power up */
					if ((rand() % 4) < 1) {
						PARTICLE *p = make_anim_particle("LaserGun");
						set_particle_property(p, PARTICLE_ALIVE | PARTICLE_WRAP | PARTICLE_COLLIDE | p->prop);
						set_particle_coord(p, asteroid->Particle->x, asteroid->Particle->y);
						set_particle_velocity(p, rand() % 40, rand() % 40);
						set_particle_rotation_speed(p, rand() % ftofix(0.05) - ftofix(0.025));
						set_particle_life(p, 1000);
						set_particle_alpha(p, 255);
						set_particle_type(p, 'l');
					}

				}
				} while (0);
				
				if (bullet == p1) {
					i--;
					reset = 1;
				}
				else
					j--;

				{
					PLAYER *p = find_player(bullet->owner);
					add_player_score(p, 5);
					destroy_particle(bullet);
				}

				if (reset)
					break;
				else
					continue;
			}

		}

	}

	/* Check if everyone's dead */
	j = 0;
	for (i = 0; i < NUM_PLAYERS; i++) {		
		if (!Game.Player[i] || !get_player_property(Game.Player[i], PLAYER_ALIVE))
			continue;
		j = 1;
	}
	if (j)
		Game.TimeToLive = 3000;

	if (Game.NumAsteroids <= 0 && Game.InitLevelSwitch == 0x7FFFFFFF) {
		play_sample(PowerUp, 255, 128, 500, FALSE);
		Game.InitLevelSwitch = 1000;
		Game.LevelSwitchDelta = 1;
		for (j = 0; j < NUM_PLAYERS; j++)
			if (Game.Player[j] && get_player_property(Game.Player[j], PLAYER_ALIVE))
				add_player_score(Game.Player[j], 20 * Game.Level);
	}

	if (Game.InitLevelSwitch > 0 && Game.InitLevelSwitch != 0x7FFFFFFF)
		Game.InitLevelSwitch -= 20;

	if (Game.NumAsteroids <= 0 && Game.InitLevelSwitch <= 0) {
		InitLevel(Game.Level + 1);		
		Game.InitLevelSwitch = 0x7FFFFFFF;
		Game.LevelSwitchDelta = 1;
	}

	return 0;
}

void DrawGame() {
	
	int i, j, m;
	PLAYER *pl;
	char playerstr[15];
	int x, y;

	clear(BackBuffer);

	for (j = 0; j < NUM_PLAYERS; j++) {

		pl = Game.Player[j];

		if (!pl)
			continue;

		if (get_player_property(pl, PLAYER_INVINCIBILITY)) {
			PARTICLE *p = get_player_particle(pl);
			if (GameTime / 50 != p->d1) {
				p->prop ^= PARTICLE_INVISIBLE;
				p->d1 = GameTime / 50;
			}
		}
		else if (get_player_property(pl, PLAYER_ALIVE)) {
			PARTICLE *p = get_player_particle(pl);
			p->prop &= ~PARTICLE_INVISIBLE;	
		}
	}

	draw_effects(BackBuffer);
	draw_stars(BackBuffer);
	draw_particles(BackBuffer);

	if (Game.LevelSwitchDelay != 0x7FFFFFFF) {
		text_mode(-1);
		textprintf_centre(BackBuffer, DigitFont, DISPLAY_W / 2, DISPLAY_H / 2 - text_height(DigitFont) / 2, makecol(255, 255, 255), "Level %02i", Game.Level);
	}
	if (Game.InitLevelSwitch != 0x7FFFFFFF) {
		text_mode(-1);
		textprintf_centre(BackBuffer, DigitFont, DISPLAY_W / 2, DISPLAY_H / 2 - text_height(DigitFont) / 2, makecol(255, 255, 255), "Level Complete !");
	}
	if (Game.LevelSwitchDelta) {
		int w = text_length(DigitFont, "Level Complete !");
		DRS_add_rectangle((DISPLAY_W - w) / 2, DISPLAY_H / 2 - text_height(DigitFont) / 2, w, text_height(DigitFont));
		Game.LevelSwitchDelta = 0;
	}

	/* Draw HUD */

	x = 0;
	y = 1;

	for (j = 0; j < NUM_PLAYERS; j++) {

		pl = Game.Player[j];

		if (!pl)
			continue;

		if (get_player_property(pl, PLAYER_ALIVE)) {

			textout(BackBuffer, font, "Shields", x, y + 27, makecol(255, 255, 255));
			
			for (i = 0; i < get_player_hp(pl) - get_player_maxhp(pl) / 2; i++)
				rectfill(BackBuffer, x + 60 + i * 44, y + 27, x + 60 + i * 44 + 42, y + 35, makecol(255, 255, 0));
			for (; i < MIN(get_player_hp(pl), get_player_maxhp(pl) / 2); i++)
				rectfill(BackBuffer, x + 60 + i * 44, y + 27, x + 60 + i * 44 + 42, y + 35, makecol(0, 255, 0));			

			if (pl->old_hp != pl->hp) {
				DRS_add_rectangle(x + 60, y + 27, 262, 9);
				pl->old_hp = pl->hp;
			}

			m = get_player_ammo_percent(pl);
			if (m > 0) {
				textout(BackBuffer, font, "Ammo", x, y + 42, makecol(255, 255, 255));
				for (i = 0; i < (m + 9) / 10; i++) {
					rectfill(BackBuffer, x + 60 + i * 22, y + 42, x + 60 + i * 22 + 20, y + 50, makecol(255, 0, 0));
				}
			}
			if (pl->old_ammo != m) {
				DRS_add_rectangle(x, y + 42, DISPLAY_W / 2 - 40,  9);
				pl->old_ammo = m;
			}

			sprintf(playerstr, "Player %i", j + 1);
			textout(BackBuffer, DigitFont, playerstr, x, y, makecol(0, 255, 0));
			textprintf_right(BackBuffer, DigitFont, x + DISPLAY_W / 2 - 40, y, makecol(0, 255, 0), "%05i", get_player_score(pl));

			if (pl->Score != pl->old_score) {
				DRS_add_rectangle(x, y, DISPLAY_W / 2 - 40, text_height(DigitFont));
				pl->old_score = pl->Score;
			}	
		}
		else {
			sprintf(playerstr, "Player %i", j + 1);
			textout(BackBuffer, DigitFont, playerstr, x, y, makecol(0, 255, 0));
			textprintf_right(BackBuffer, DigitFont, x + DISPLAY_W / 2 - 40, y, makecol(0, 255, 0), "%05i", get_player_score(pl));
			textprintf(BackBuffer, DigitFont, x, y + text_height(DigitFont), makecol(192, 0, 0), "Game Over");

			if (pl->old_prop != pl->Properties) {
				DRS_add_rectangle(x, y, DISPLAY_W / 2 - 40, text_height(DigitFont) * 3);
				pl->old_prop = pl->Properties;
			}
		}
		x += DISPLAY_W / 2;
			
		if (x >= SCREEN_W) {
			x = 0;
			y += 60;
		}
	}

	if (GamePaused) {
		int w = text_length(DigitFont, "Game Paused");
		textout_centre(BackBuffer, DigitFont, "Game Paused", DISPLAY_W / 2, DISPLAY_H / 2 + text_height(DigitFont), makecol(255, 255, 255));
		DRS_add_rectangle((DISPLAY_W - w) / 2, DISPLAY_H / 2 + text_height(DigitFont), w, text_height(DigitFont));
	}
}

int game_input(void *dummy) {

	PLAYER *pl;
	int j;
	PARTICLE *p;

	poll_joystick();

	if (key[KEY_F12]) {
		clear_keybuf();
		GamePaused ^= 1;
	}

	if (GamePaused)
		return 0;

	for (j = 0; j < NUM_PLAYERS; j++) {

		
		pl = Game.Player[j];

		if (!pl)
			continue;

		p = get_player_particle(pl);

		if (!get_player_property(pl, PLAYER_ALIVE))
			continue;
	
		if (player_get_input_left(pl)) {
			set_particle_rotation_speed(p, itofix(-200));
		}
		else if (player_get_input_right(pl)) {
			set_particle_rotation_speed(p, itofix(200));
		}
		else 
			set_particle_rotation_speed(p, 0);

		if (player_get_input_up(pl)) {
			float d;
			float angle = get_particle_angle(p);

			p->vx += sin(angle) * 40;
			p->vy += -cos(angle) * 40;

			/* Max out speed */
			d = DIST(p->vx, p->vy);
			if (d > 300) {
				p->vx *= 300 / d;
				p->vy *= 300 / d;
			}		
		}
		else if (player_get_input_down(pl)) {
			p->vx /= 1.5;
			p->vy /= 1.5;		
		}					

		if (player_get_input_fire(pl)) {
			/* shoot */
			player_fire(pl);
		}
	}

	return 0;
}

void DoGame(int n) {

	int i, k;
	int frame_count = 0;
	int frame_start_time = 0;
	float frame_rate = 0;
	int frame_timer = -1;

	LOG("Starting new game for %i players\n", n);

	for (i = 0; i < n; i++) {
		LOG("Preparing player % i\n", i);

		Game.Player[i] = create_player();
		set_player_bitmap(Game.Player[i], PlayerBitmap[i]);

		if (n == 1)
			set_player_coord(Game.Player[i], DISPLAY_W / 2, DISPLAY_H / 2);
		else
			set_player_coord(Game.Player[i], DISPLAY_W / 2 + cos(i * M_PI * 2 / n + M_PI) * DISPLAY_W / 6, DISPLAY_H / 2 - sin(i * M_PI * 2 / n + M_PI) * DISPLAY_H / 6);

		set_player_hp(Game.Player[i], 5, 10);
		set_player_property(Game.Player[i], PLAYER_ALIVE);
		remove_all_player_weapons(Game.Player[i]);

		add_player_weapon(Game.Player[i], NormalGun);	

		load_player_input(Game.Player[i], i);
	}
	for (i = n; i < NUM_PLAYERS; i++)
		Game.Player[i] = NULL;

	Game.InitLevelSwitch = 0x7FFFFFFF;

	play_sample(Engine, 1, 128, 1000, TRUE);

	LOG("Initializing level 1\n");

	for (i = 0; i < 60; i++) {
		int c = rand() % 64 + 192;
		create_star(rand() % DISPLAY_W, rand() % DISPLAY_H, makecol(c, c, c));
	}

	InitLevel(1);

	LOG("Hooking timers\n");
	GTIME_AddFunction(play_game, 20, NULL, GTIME_NORMAL);
	GTIME_AddFunction(game_input, 50, NULL, GTIME_NORMAL);

	LOG("Starting game...NOW!\n");
	
	do {
		DRS_start_frame();

		GTIME_Poll();

		DrawGame();

		if (ShowFPS) {
			frame_count++;
			if ((frame_count & 31) == 0) {
				frame_rate = frame_count * 1000.0 / ((float)GameTime - frame_start_time);
				frame_start_time = GameTime;
				frame_count = 0;
			}
			textprintf(BackBuffer, font, 0, SCREEN_H - 10, makecol(255, 255, 255), "FPS: %.2f", frame_rate);
			DRS_add_rectangle(0, SCREEN_H - 10, 128, 8);
		}
		else {
			vsync();
		}

		acquire_screen();

		if (frame_timer != GameTime / 1000) {
			frame_timer = GameTime / 1000;
			DRS_restart();
		}
		/*
		if (DISPLAY_W != SCREEN_W || DISPLAY_H != SCREEN_H)
			stretch_blit(BackBuffer, screen, 0, 0, BackBuffer->w, BackBuffer->h, 0, 0, SCREEN_W, SCREEN_H);
		else		
			blit(BackBuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);			
		*/
		DRS_update_screen();

		release_screen();

		if (Game.TimeToLive <= 0)
			break;

	} while (!key[KEY_ESC]);

	LOG("Game ended\n");

	stop_sample(Engine);

	LOG("Removing timers\n");
	GTIME_RemoveFunction(play_game, NULL);
	GTIME_RemoveFunction(game_input, NULL);

	LOG("Processing scores.\n");
	play_sample(Tick, 255, 128, 1000, FALSE);
	k = 0;
	for (i = 0; i < n; i++) {
		k |= (ProcessHighScore(get_player_score(Game.Player[i]), i + 1) >= 0) ? 1 : 0;
	}
	if (k) {
		play_sample(Tick, 255, 128, 1000, FALSE);
		ViewHighScore();
	}
	play_sample(Tick, 255, 128, 1000, FALSE);

	LOG("Flushing old asteroid data\n");

	/* flush old data */
	for (i = 0; i < Game.NumAsteroids; i++)
		destroy_asteroid(Game.Asteroid[i]);

	if (Game.Asteroid)
		free(Game.Asteroid);
	Game.Asteroid = NULL;

	LOG("Flushing old player data\n");
	for (i = 0; i < n; i++)
		remove_all_player_weapons(Game.Player[i]);
	clear_players();

	LOG("Flushing stars\n");
	clear_stars();
	LOG("Flushing particles\n");
	clear_particles();

	return;
}


void SwitchCallback(void) {

	DRS_restart();

	return;
}


#ifdef FORTIFY
void a_Fortify_LeaveScope(void) {
	Fortify_LeaveScope();
}
#endif


int main(int argc, char *argv[]) {

	int r;
	int w, h, c, mvol, muvol;

    
    #ifdef FORTIFY
		Fortify_SetOutputFunc(LOG);

		Fortify_EnterScope();

		atexit(a_Fortify_LeaveScope);
    #endif

#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif


	allegro_init();	

	set_config_file("zasx.cfg");
	w = get_config_int("[screen]", "width", 640);
	h = get_config_int("[screen]", "height", 480);
	c = get_config_int("[screen]", "color", desktop_color_depth());
	ShowFPS = get_config_int("[screen]", "show_fps", 0);
	mvol = get_config_int("[sound]", "master_volume", 255);
	muvol = get_config_int("[sound]", "music_volume", 255);
	

	/* Log init */
	LOG("\n\n\n");
	LOG("******************************************\n");
	LOG("*\n");
	LOG("*  Zasx  v" VERSION "\n");
	LOG("*\n");
	LOG("******************************************\n");


	/* Init */
	GFX_Init(w, h, c);
	set_window_title("Zasx v" VERSION);
	set_display_switch_callback(SWITCH_IN, SwitchCallback);


	install_keyboard();
	install_joystick(JOY_TYPE_AUTODETECT);
	install_timer();
	reserve_voices(16, -1);

	install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL);
	//install_mod(12);
	set_volume(mvol, -1);

	DRS_init(2000, NULL, 0);
	BackBuffer = DRS_work;

	srand(time(NULL));

	set_keyboard_rate(0, 0);


	GTIME_Install();

	if ((r = LoadData()))
		return r;
	

	install_particle();
	install_effects();
	install_star();
	install_player();

	//set_mod_volume(muvol);
	//play_mod(Music, TRUE);

	DoMenu();

	DRS_exit();

	//stop_mod();

	clear_particles();

	UnloadData();

	GFX_Shutdown();

#ifdef FORTIFY
	Fortify_OutputStatistics();
#endif

	return 0;
} END_OF_MAIN();

