
#include <stdio.h>
#include <string.h> 
#include <time.h>

#include "allegro.h"
#include "game.h"
#include "gfx.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif


static void DrawBG() {
	int i;

	clear(BackBuffer);

	srand(0);
	for (i = 0; i < 400; i++) {
		int c = rand() % 64 + 192;
		putpixel(BackBuffer, rand() % DISPLAY_W, rand() % DISPLAY_H, makecol(c, c, c));
	}
	srand(time(NULL));
	return;
}

void ShowCredits() {

	DrawBG();

	textout(BackBuffer, font, " Programming, Design, and most Graphics:", 0, 30, makecol(255, 255, 255));
	textout(BackBuffer, font, "   Robert J Ohannessian", 0, 40, makecol(255, 255, 255));

	textout(BackBuffer, font, " Special thanks to:", 0, 70, makecol(255, 255, 255));
	textout(BackBuffer, font, "   Shawn Hargreaves, (Allegro, the asteroid graphics and some sound effects)", 0, 80, makecol(255, 255, 255));
	textout(BackBuffer, font, "   Ivan Baldo, (PPCol)", 0, 90, makecol(255, 255, 255));
	textout(BackBuffer, font, "   Jeffery Guan, (JGMOD)", 0, 100, makecol(255, 255, 255));
	textout(BackBuffer, font, "   Matthew Leverton (allegro.cc)", 0, 110, makecol(255, 255, 255));
	textout(BackBuffer, font, "   The whole Allegro team!", 0, 120, makecol(255, 255, 255));


	textprintf_centre(BackBuffer, font, DISPLAY_W / 2, DISPLAY_H - text_height(font), makecol(255, 255, 255), "Press any key to continue");

	stretch_blit(BackBuffer, screen, 0, 0, BackBuffer->w, BackBuffer->h, 0, 0, SCREEN_W, SCREEN_H);

	while (keypressed()) readkey();
	readkey();

	return;
}

static void DrawScore(int p, int this_score, int player) {

	int i;
	int color = makecol(255, 255, 255);

	text_mode(-1);
	if (player == -1) 
		textout_centre(BackBuffer, DigitFont, "Top Scores", DISPLAY_W / 2, 14, color);
	else
		textprintf_centre(BackBuffer, DigitFont, DISPLAY_W / 2, 14, color, "Enter name for Player %i", player);

	for (i = 0; i < 10; i++) {
		char t[80];
		const char *best_name;
		int score;

		sprintf(t, "Name%0i", i);
		best_name = get_config_string("[scores]", t, "Bob");
		sprintf(t, "Score%0i", i);
		score = get_config_int("[scores]", t, 0);

		textprintf(BackBuffer, DigitFont, 5, i * (text_height(DigitFont) + 3) + 50, color, "%2i.", i + 1);
		textprintf_right(BackBuffer, DigitFont, DISPLAY_W - 10, i * (text_height(DigitFont) + 3) + 50, color, "%01i", (i == p) ? this_score : score);

		if (i == p)
			rect(BackBuffer, 3, i * (text_height(DigitFont) + 3) + 48, DISPLAY_W - 8, i * (text_height(DigitFont) + 3) + 52 + text_height(DigitFont), makecol(0, 255, 0));
		else
			textprintf(BackBuffer, DigitFont, 5 + text_length(DigitFont, " 0. "), i * (text_height(DigitFont) + 3) + 50, color, "%s", best_name);
	}
	return;
}


void ViewHighScore() {

	push_config_state();
	set_config_file("data/zasx.hsc");

	DrawBG();
	DrawScore(-1, -1, -1);

	pop_config_state();

	textprintf_centre(BackBuffer, font, DISPLAY_W / 2, DISPLAY_H - text_height(font), makecol(255, 255, 255), "Press any key to continue");

	stretch_blit(BackBuffer, screen, 0, 0, BackBuffer->w, BackBuffer->h, 0, 0, SCREEN_W, SCREEN_H);

	while (keypressed()) readkey();
	readkey();

	return;
}

int my_edit_proc(int msg, DIALOG *d, int c)
{
   static int ignore_next_uchar = FALSE;
   int f, l, p, w, x, fg, b, scroll;
   char buf[16];
   char *s;
   int rtm;

   s = d->dp;
   l = ustrlen(s);

   if (d->d2 > l) 
      d->d2 = l;

   /* calculate maximal number of displayable characters */
   if (d->d2 == l)  {
      usetc(buf+usetc(buf, ' '), 0);
      x = text_length(DigitFont, buf);
   }
   else
      x = 0;

   b = 0;

   if (l) {
	for (p=d->d2; p>=0; p--) {
		usetc(buf+usetc(buf, ugetat(s, p)), 0);
		x += text_length(DigitFont, buf);
		b++;
		if (x > d->w) 
			break;
	}
   }

   if (x <= d->w) {
      b = l; 
      scroll = FALSE;
   }
   else {
      b--; 
      scroll = TRUE;
   }

   switch (msg) {

      case MSG_START:
		  {
		  BITMAP *temp = create_bitmap(d->w, d->h);
		  blit(BackBuffer, temp, d->x, d->y, 0, 0, d->w, d->h);
		  d->dp3 = temp;
		  }
			break;

	  case MSG_END:
		  if (d->dp3)
			  destroy_bitmap(d->dp3);
		  break;

      case MSG_DRAW:
	 fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
	 x = 0;

	 if (scroll) {
	    p = d->d2-b+1; 
	    b = d->d2; 
	 }
	 else 
	    p = 0; 

	 blit((BITMAP*)d->dp3, BackBuffer, 0, 0, d->x, d->y, d->w, d->h);

	 for (; p<=b; p++) {
	    f = ugetat(s, p);
	    usetc(buf+usetc(buf, (f) ? f : ' '), 0);
	    w = text_length(DigitFont, buf);
	    if (x+w > d->w) 
	       break;
	    f = ((p == d->d2) && (d->flags & D_GOTFOCUS));
		rtm = text_mode(f ? fg : -1);
		textout(BackBuffer, DigitFont, buf, d->x+x, d->y, fg);
	    text_mode(rtm);
	    x += w;
	 }
	 stretch_blit(BackBuffer, screen, 0, 0, BackBuffer->w, BackBuffer->h, 0, 0, SCREEN_W, SCREEN_H);
	 break;

      case MSG_WANTFOCUS:
      case MSG_LOSTFOCUS:
      case MSG_KEY:
	 return D_WANTFOCUS;

      case MSG_CHAR:
	 ignore_next_uchar = FALSE;

	 if ((c >> 8) == KEY_LEFT) {
	    if (d->d2 > 0) d->d2--;
	 }
	 else if ((c >> 8) == KEY_RIGHT) {
	    if (d->d2 < l) d->d2++;
	 }
	 else if ((c >> 8) == KEY_HOME) {
	    d->d2 = 0;
	 }
	 else if ((c >> 8) == KEY_END) {
	    d->d2 = l;
	 }
	 else if ((c >> 8) == KEY_DEL) {
	    if (d->d2 < l)
	       uremove(s, d->d2);
	 }
	 else if ((c >> 8) == KEY_BACKSPACE) {
	    if (d->d2 > 0) {
	       d->d2--;
	       uremove(s, d->d2);
	    }
	 }
	 else if ((c >> 8) == KEY_ENTER) {
	    if (d->flags & D_EXIT) {
	       scare_mouse();
	       SEND_MESSAGE(d, MSG_DRAW, 0);
	       unscare_mouse();
	       return D_CLOSE;
	    }
	    else
	       return D_O_K;
	 }
	 else if ((c >> 8) == KEY_TAB) {
	    ignore_next_uchar = TRUE;
	    return D_O_K;
	 }
	 else {
	    /* don't process regular keys here: MSG_UCHAR will do that */
	    break;
	 }
	 scare_mouse();
	 SEND_MESSAGE(d, MSG_DRAW, 0);
	 unscare_mouse();
	 return D_USED_CHAR;

      case MSG_UCHAR:
	 if ((c >= ' ') && (uisok(c)) && (!ignore_next_uchar)) {
	    if (l < d->d1) {
	       uinsert(s, d->d2, c);
	       d->d2++;

	       scare_mouse();
	       SEND_MESSAGE(d, MSG_DRAW, 0);
	       unscare_mouse();
	    }
	    return D_USED_CHAR;
	 }
	 break;
   }

   return D_O_K;
}


int ProcessHighScore(int this_score, int player) {

	int i, p = -1;

	push_config_state();
	set_config_file("data/zasx.hsc");

	for (i = 0; i < 10; i++) {
		char t[80];
		const char *best_name;
		int score;

		sprintf(t, "Name%0i", i);
		best_name = get_config_string("[scores]", t, "Bob");
		sprintf(t, "Score%0i", i);
		score = get_config_int("[scores]", t, 0);

		if (this_score > score) {
			p = i;
			break;
		}
	}

	if (p >= 0) {
		/* Move everyone down */
		const char *name;
		int score;
		char t[80];

		for (i = 8; i >= p; i--) {
			
			sprintf(t, "Name%0i", i);
			name = get_config_string("[scores]", t, "Bob");
			sprintf(t, "Score%0i", i);
			score = get_config_int("[scores]", t, 0);

			sprintf(t, "Name%0i", i + 1);
			set_config_string("[scores]", t, name);
			sprintf(t, "Score%0i", i + 1);
			set_config_int("[scores]", t, score);
		}

		/* Ask for name */
		DrawBG();
		DrawScore(p, this_score, player);
		
		{
			char name[80];
			DIALOG d[] = {
				{my_edit_proc, 5 + text_length(DigitFont, " 0. "), p * (text_height(DigitFont) + 3) + 50, 500, text_height(DigitFont), makecol(255, 255, 255), 0, 0, D_EXIT, 22, 0, name, NULL, NULL },
				{NULL}
			};
			memset(name, 0, 80);
			stretch_blit(BackBuffer, screen, 0, 0, BackBuffer->w, BackBuffer->h, 0, 0, SCREEN_W, SCREEN_H);

			while (keypressed())
				readkey();

			text_mode(-1);
			do_dialog(d, 0);

			if (!ustrlen(name))
				name[0] = '_';

			sprintf(t, "Name%0i", p);
			set_config_string("[scores]", t, name);
			sprintf(t, "Score%0i", p);
			set_config_int("[scores]", t, this_score);
		}
	}

	pop_config_state();

	return p;
}
