
#include <string.h>

#include "allegro.h"
#include "particle.h"
#include "star.h"
#include "gametimer.h"
#include "gfx.h"
#include "game.h"
#include "drs.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif


STAR **Star = NULL;
int NumStars = 0;
static int StarArraySize = 0;
static int StarInstalled = 0;

RAY **Ray = NULL;
int NumRays = 0;
static int RayArraySize = 0;


static void remove_ray(RAY *e);
static void destroy_ray(RAY *ray);

static void add_star(STAR *s) {

	if (NumStars - StarArraySize <= 1) {
		Star = realloc(Star, sizeof(STAR*) * (StarArraySize + 16));
		StarArraySize += 16;
	}

	Star[NumStars] = s;

	NumStars++;

	return;
}

static void remove_star(STAR *e) {

	int i;

	for (i = 0; i < NumStars; i++) {
		if (Star[i] == e) {		
			Star[i] = Star[NumStars - 1];
			NumStars--;
			break;
		}
	}

	return;
}


STAR *create_star(int x, int y, int color) {

	STAR *s = malloc(sizeof(STAR));

	if ((ABS(x - DISPLAY_W / 2) < 10) && (x > DISPLAY_W / 2))
		x += 5;
	if ((ABS(y - DISPLAY_H / 2) < 10) && (y > DISPLAY_H / 2))
		y += 5;
	if ((ABS(x - DISPLAY_W / 2) < 10) && (x <= DISPLAY_W / 2))
		x -= 5;
	if ((ABS(y - DISPLAY_H / 2) < 10) && (y <= DISPLAY_H / 2))
		y -= 5;

	s->x = itofix(x);
	s->y = itofix(y);

	s->vx = s->x - itofix(DISPLAY_W / 2);
	s->vy = s->y - itofix(DISPLAY_H / 2);
	s->color = color;

	s->r = getr(color);
	s->g = getg(color);
	s->b = getb(color);

	add_star(s);

	return s;
}

#define SWAP(a, b) { \
	(a) ^= (b);      \
	(b) ^= (a);      \
	(a) ^= (b);      \
}


void line_drs_callback(BITMAP *bmp, int x, int y, int d) {
	DRS_add_rectangle(x, y, 1, 1);
}

void draw_stars(BITMAP *dest) {
	int i;
	int x1, y1, x2, y2;

	for (i = 0; i < NumRays; i++) {		
		RAY *r = Ray[i];

		x1 = r->x1 / r->z1 + DISPLAY_W / 2;
		y1 = r->y1 / r->z1 + DISPLAY_H / 2;
		x2 = r->x2 / r->z2 + DISPLAY_W / 2;
		y2 = r->y2 / r->z2 + DISPLAY_H / 2;
		
		line(dest, x1, y1, x2, y2, r->color1);

		do_line(dest, x1, y1, x2, y2, 0, line_drs_callback);
		do_line(dest, r->old_x1, r->old_y1, r->old_x2, r->old_y2, 0, line_drs_callback);

		r->old_x1 = x1;
		r->old_x2 = x2;
		r->old_y1 = y1;
		r->old_y2 = y2;
	}

	for (i = 0; i < NumStars; i++) {
		STAR *s = Star[i];
		int x, y;
		int fact, col;

		x = fixtoi(s->x);
		y = fixtoi(s->y);
		fact = ABS(2 * x - DISPLAY_W) * 256 / DISPLAY_W + ABS(2 * y - DISPLAY_H) * 256 / DISPLAY_H;
		fact = MID(128, fact, 255);

		col = makecol(s->r * fact >> 8, s->g * fact >> 8, s->b * fact >> 8);

		putpixel(dest, x, y, col);

		DRS_add_rectangle(x, y, 1, 1);
		DRS_add_rectangle(s->old_x, s->old_y, 1, 1);
		s->old_x = x;
		s->old_y = y;
	}

	return;
}


void clear_stars() {
	int i = 0;

	while (NumStars)
		destroy_star(Star[0]);

	if (Star)
		free(Star);

	NumStars = 0;
	Star = NULL;
	StarArraySize = 0;

	while (NumRays)
		destroy_ray(Ray[0]);

	if (Ray)
		free(Ray);

	NumRays = 0;
	Ray = NULL;
	RayArraySize = 0;

	return;
}

void destroy_star(STAR *star) {

	if (!star)
		return;

	remove_star(star);

	free(star);
}


void destroy_ray(RAY *ray) {

	if (!ray)
		return;

	do_line(DRS_work, ray->old_x1, ray->old_y1, ray->old_x2, ray->old_y2, 0, line_drs_callback);

	remove_ray(ray);

	free(ray);
}


void add_ray() {

	int r, g, b;

	RAY *ray = malloc(sizeof(RAY));
	memset(ray, 0, sizeof(RAY));

	ray->x1 = rand() % DISPLAY_W - DISPLAY_W / 2;
	ray->x2 = ray->x1;
	ray->y1 = rand() % DISPLAY_H - DISPLAY_H / 2;
	ray->y2 = ray->y1;

	ray->z1 = 20;
	ray->z2 = 25;


	hsv_to_rgb(rand() % 360, 1, 1, &r, &g, &b);
	ray->color1 = makecol(r, g, b);
	ray->color2 = makecol(r / 2, g / 2, b / 2);

	if (NumRays - RayArraySize <= 1) {
		Ray = realloc(Ray, sizeof(RAY*) * (RayArraySize + 16));
		RayArraySize += 16;
	}

	Ray[NumRays] = ray;
	NumRays++;

	return;
}

static void remove_ray(RAY *e) {

	int i;

	for (i = 0; i < NumRays; i++) {
		if (Ray[i] == e) {
			Ray[i] = Ray[NumRays - 1];
			NumRays--;
			break;
		}
	}

	return;
}

void line_intersect(float x0,float y0,float x1,float y1, float x2,float y2,float x3,float y3, float *xi,float *yi) {

	float a1,b1,c1, a2,b2,c2,
		det_inv,
		m1,m2;
	
	if ((x1-x0)!=0)
		m1 = (y1-y0)/(x1-x0);
	else
		m1 = (float)1e+10;
	if ((x3-x2)!=0)
		m2 = (y3-y2)/(x3-x2);
	else
		m2 = (float)1e+10;

	a1 = m1;
	a2 = m2; 

	b1 = -1;
	b2 = -1;

	c1 = (y0 - m1 * x0);
	c2 = (y2 - m2 * x2); 

	det_inv = 1.0 / (a1 * b2 - a2 * b1); 

	*xi=((b1 * c2 - b2 * c1) * det_inv);
	*yi=((a2 * c1 - a1 * c2) * det_inv);
}

int process_stars(void *dummy) {

	int i;

	if (GamePaused)
		return 0;

	for (i = 0; i < NumStars; i++) {
		STAR *s = Star[i];

		s->x += s->vx / 100;
		s->y += s->vy / 100;

		if (s->x < 0 || s->x > itofix(DISPLAY_W) || s->y < 0 || s->y > itofix(DISPLAY_H)) {
			s->x = itofix(DISPLAY_W / 2);
			s->y = itofix(DISPLAY_H / 2);
		}
	}

	for (i = 0; i < NumRays; i++) {
		RAY *r = Ray[i];
		r->z1 -= .3;
		r->z2 -= .3;

		if (r->z1 <= 0.01 && r->z2 <= 0.01) {
			destroy_ray(r);
			i--;
			continue;
		}
		else if (r->z1 <= 0.01) {
			r->z1 += .3;
			r->x1 /= r->z1;
			r->y1 /= r->z1;
			r->x2 /= r->z2;
			r->y2 /= r->z2;

			if (r->x1 > DISPLAY_W / 2)
				line_intersect(r->x1, r->y1, r->x2, r->y2,  DISPLAY_W / 2, -DISPLAY_H / 2,  DISPLAY_W / 2,  DISPLAY_H / 2, &r->x1, &r->y1);			
			if (r->x1 < -DISPLAY_W / 2)				
				line_intersect(r->x1, r->y1, r->x2, r->y2, -DISPLAY_W / 2, -DISPLAY_H / 2, -DISPLAY_W / 2,  DISPLAY_H / 2, &r->x1, &r->y1);
			if (r->y1 < -DISPLAY_H / 2)
				line_intersect(r->x1, r->y1, r->x2, r->y2, -DISPLAY_W / 2, -DISPLAY_H / 2,  DISPLAY_W / 2, -DISPLAY_H / 2, &r->x1, &r->y1);
			if (r->y1 > DISPLAY_H / 2)
				line_intersect(r->x1, r->y1, r->x2, r->y2, -DISPLAY_W / 2,  DISPLAY_H / 2,  DISPLAY_W / 2,  DISPLAY_H / 2, &r->x1, &r->y1);			

			r->x1 *= r->z1;
			r->y1 *= r->z1;
			r->x2 *= r->z2;
			r->y2 *= r->z2;

		}
	}
	return 0;
}

int install_star() {

	if (StarInstalled)
		return 0;

	GTIME_AddFunction(process_stars, 10, NULL, GTIME_NORMAL);

	StarInstalled = 1;

	return 0;
}
