

#include <math.h>
#include <string.h>
#include <stdio.h>

#include "allegro.h"
#include "particle.h"
#include "weapon.h"
#include "log.h"
#include "game.h"

#ifdef DEBUGMODE
#include "fortify.h"
#endif

#ifndef M_PI
	#define M_PI 3.14159265359
#endif


WEAPON *create_weapon() {

	WEAPON *w = malloc(sizeof(WEAPON));

	if (!w)
		return NULL;

	memset(w, 0, sizeof(WEAPON));

	return w;
}

void destroy_weapon(WEAPON *w) {

	int i, j;

	if (!w)
		return;

	for (j = 0; j < w->NumCannons; j++) {

		WEAPON_CANNON *c = w->Cannon[j];

		if (c && c->Image) {
	
			for (i = 0; i < c->NumImages; i++) 
				if (c->Image[i])
					destroy_bitmap(c->Image[i]);
	
			free(c->Image);
		}
		if (c)
			free(c);
	}

	if (w->Cannon)
		free(w->Cannon);

	free(w);

	return;
}


static void strip_leading_whitespace(char *buf) {


	char *cur = buf;

	if (!buf)
		return;

	//sscanf(buf, "%[^\n]", buf);

	return;
}

static void dup_sample(SAMPLE **dest, SAMPLE *src) {

	SAMPLE *d;

	if (!dest)
		return;

	if (!src) {
		*dest = NULL;
		return;
	}

	d = create_sample(src->bits, src->stereo, src->freq, src->len);

	*dest = d;

	if (!d)
		return;

	memcpy(d->data, src->data, (src->bits / 8) * (src->stereo + 1) * src->len);

	return;
}


static void add_cannon(WEAPON *w, WEAPON_CANNON *c) {

	WEAPON_CANNON **wc;

	if (!w || !c)
		return;

	wc = realloc(w->Cannon, sizeof(WEAPON_CANNON*) * (w->NumCannons + 1));

	if (!wc) {
		LOG("Error allocating mem!\n");
		return;
	}

	w->Cannon = wc;
	w->Cannon[w->NumCannons] = c;
	w->NumCannons++;

	return;
}


WEAPON *load_weapon(char *filename) {

	PACKFILE *p;
	WEAPON *w = NULL;
	WEAPON_CANNON *c = NULL;
	char buf[520];
	char buf2[520];

	if (!filename)
		return NULL;


	LOG("Loading %s\n", filename);

	p = pack_fopen(filename, "rp");

	if (!p) {
		p = pack_fopen(filename, "r");

		if (!p) {
			LOG("Unable to open %s\n", filename);
			return NULL;
		}
	}

	GamePal[0].r =  63; GamePal[0].g =  0; GamePal[0].b =  63;
	select_palette(GamePal);

	while(pack_fgets(buf, 512, p)) {
		int r, i;
		float x, y;

		strip_leading_whitespace(buf);

		if (buf[0] == '#')
			continue;

		r = strcmp(buf, "[Weapon]");

		if (!r) {
			if (w)
				destroy_weapon(w);

			w = create_weapon();
			continue;
		}

		r = strcmp(buf, "[Cannon]");		

		if (!r) {
			if (w) {
				c = malloc(sizeof(WEAPON_CANNON));
				if (c) {
					memset(c, 0, sizeof(WEAPON_CANNON));
					add_cannon(w, c);
				}
			}
			continue;
		}

		r = sscanf(buf, "Coord=%f %f", &x, &y);

		if (r > 0 && w && !c) {
			w->x = x;
			w->y = y;
			continue;
		}
		else if (r > 0 && c) {
			c->x = x;
			c->y = y;
			continue;
		}

		r = sscanf(buf, "Velocity=%f %f", &x, &y);

		if (r > 0 && c) {
			c->vx = x;
			c->vy = y;
			continue;
		}

		r = sscanf(buf, "Alpha=%i", &i);

		if (r > 0 && c) {
			c->Alpha = i;
			continue;
		}

		r = sscanf(buf, "AlphaSpeed=%i", &i);

		if (r > 0 && c) {
			c->AlphaSpeed = i;
			continue;
		}

		r = sscanf(buf, "ShotDelay=%i", &i);

		if (r > 0 && w) {
			w->ShootDelay = i;
			continue;
		}

		r = sscanf(buf, "Life=%i", &i);

		if (r > 0 && c) {
			c->Life = i;
			continue;
		}

		r = sscanf(buf, "Ammo=%i", &i);

		if (r > 0 && w) {
			w->Ammo = i;
			continue;
		}

		r = sscanf(buf, "Power=%f", &x);

		if (r > 0 && c) {
			c->Power = ftofix(x);
			continue;
		}

		r = sscanf(buf, "NumImages=%i", &i);

		if (r > 0 && c) {
			c->NumImages = i;
			c->Image = malloc(sizeof(BITMAP*) * i);
			continue;
		}

		r = sscanf(buf, "Images=%[^\n]", buf2);

		if (r > 0 && c) {
			char *temp = buf;			
			PALETTE pal;
			char *g;


			strcpy(buf, buf2);

			/* Parse the string, ';' delimited file names */
			for (i = 0; i < c->NumImages; i++) {

				c->Image[i]= NULL;

				sscanf(temp, "%[^ \n\t;]", buf2);

				/* Attempt to load it as a datafile object */
				if ((g = strstr(buf2, "#"))) {
					DATAFILE *d;
					BITMAP *b = NULL;
					char oldg;


					oldg = *g;
					*g = '\0';
					d = load_datafile_object(buf2, g + 1);
					*g = oldg;

					if (!d) {
						LOG("Error loading the datafile. Trying direct bitmap reading\n");
						c->Image[i] = load_bitmap(buf2, pal);
					}
					else {
					
						if (d->type == DAT_BITMAP) {
							b = convert_bitmap((BITMAP*)d->dat, GamePal);
						}
						else if (d->type == DAT_RLE_SPRITE) {
							b = create_bitmap(((RLE_SPRITE*)d->dat)->w, ((RLE_SPRITE*)d->dat)->h);
		
							if (b) {
								clear_to_color(b, b->vtable->mask_color);
								draw_rle_sprite(b, ((RLE_SPRITE*)d->dat), 0, 0);
							}
						}
						unload_datafile_object(d);

						c->Image[i] = b;
					}
				}
				if (!c->Image[i]) {
					c->Image[i] = load_bitmap(buf2, pal);
				}

				temp += strlen(buf2) + 1;
			}

			continue;
		}

		r = sscanf(buf, "Sound=%[^\n]", buf2);

		if (r > 0 && w) {
			SAMPLE *sam = NULL;
			DATAFILE *d;
			char oldg;
			char *g;

			g = strstr(buf2, "#");
			if (g) {
				oldg = *g;
				*g = '\0';
			
				d = load_datafile_object(buf2, g + 1);

				*g = oldg;

				if (!d) {
					LOG("Error loading the datafile. Trying direct wave reading\n");					
					w->Sound = load_sample(buf2);
				}
				else {

					if (d->type == DAT_SAMPLE)
						dup_sample(&sam, d->dat);
			
					unload_datafile_object(d);

					w->Sound = sam;
				}
			}
			if (!w->Sound) {
				w->Sound = load_sample(buf2);
			}			

			continue;
		}

		r = sscanf(buf, "Properties=%[^\n]", buf2);

		if (r > 0 && c) {
			char *temp = buf2;
			char buf3[128];

			/* Parse the string, ';' delimited properties */
			while (temp < buf2 + strlen(buf2)) {

				sscanf(temp, "%[^ \n\t;]", buf3);


				if (strcmp(buf3, "ALPHA") == 0)
					c->Properties |= PARTICLE_ALPHA;

				temp += strlen(buf3) + 1;
			}
			continue;
		}
	}

	pack_fclose(p);

	unselect_palette();
	GamePal[0].r = 0; GamePal[0].g = 0;  GamePal[0].b = 0; 


	return w;
}

void set_weapon_next_shot_delay(SHIP_WEAPON *s, int val) {

	if (!s)
		return;

	s->ShootDelay = val;

	return;
}

int get_weapon_next_shot_delay(SHIP_WEAPON *s) {
	return (s ? s->ShootDelay : 0);
}


int get_ship_weapon_ammo(SHIP_WEAPON *w) {

	if (!w)
		return 0;

	return (w->Ammo < 0) ? 0 : w->Ammo;
}

int get_weapon_ammo(WEAPON *w) {

	if (!w)
		return 0;

	return (w->Ammo < 0) ? 0: w->Ammo;
}


void weapon_fire(SHIP_WEAPON *s, PARTICLE *p) {

	WEAPON *w;
	PARTICLE *a;
	float x, y, vx, vy;
	int ph, i;
	float pos_rad, vel_rad;
	float pos_angle, vel_angle;

	if (!s || !s->Weapon || !s->Ammo)
		return;

	w = s->Weapon;

	for (i = 0; i < w->NumCannons; i++) {

		WEAPON_CANNON *c = w->Cannon[i];

		ph = get_particle_current_frame(p) ? get_particle_bitmap_h(p) : 0;
		pos_angle = atan2(w->y + c->y, w->x + c->x) + get_particle_angle(p) + M_PI;
		pos_rad = sqrt(((w->x + c->x) * (w->x + c->x)) + ((w->y + c->y) * (w->y + c->y))) * ph;
		vel_angle = get_particle_angle(p) + atan2(c->vy, c->vx) + M_PI;
		vel_rad = sqrt((c->vx * c->vx) + (c->vy * c->vy));

		a = create_particle();
	
		get_particle_coord(p, &x, &y);
		get_particle_velocity(p, &vx, &vy);

		set_particle_property(a, PARTICLE_ALIVE | PARTICLE_WRAP | PARTICLE_ROTATES | ((c->Properties & PARTICLE_ALPHA) ? (PARTICLE_ALPHA_DIMINISH | PARTICLE_ALPHA) : 0) | PARTICLE_COLLIDE);
		set_particle_coord(a, x + cos(pos_angle) * pos_rad / 2, y + sin(pos_angle) * pos_rad / 2);
		set_particle_velocity(a,  cos(vel_angle) * vel_rad + vx,    sin(vel_angle) * vel_rad + vy);
		set_particle_life(a, c->Life);

		add_particle_bitmap(a, c->Image ? c->Image[0] : NULL);

		set_particle_alpha(a, c->Alpha);
		a->alpha_speed = c->AlphaSpeed;
		set_weapon_next_shot_delay(s, w->ShootDelay);
		set_particle_angle(a, vel_angle + M_PI / 2);
		set_particle_type(a, 'B');
		set_particle_power(a, c->Power);
		set_particle_owner(a, p);
	}
	
	if (s->Ammo > 0)
		s->Ammo--;

	if (w->Sound)
		play_sample(w->Sound, 255, 128, 1000, FALSE);

	return;
}
